# Copyright 2017 GohighSec.
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import itertools

from cloudkitty.common.policies import base
from cloudkitty.common.policies import collector
from cloudkitty.common.policies import info
from cloudkitty.common.policies import rating
from cloudkitty.common.policies import report
from cloudkitty.common.policies import storage


def list_rules():
    return itertools.chain(
        base.list_rules(),
        collector.list_rules(),
        info.list_rules(),
        rating.list_rules(),
        report.list_rules(),
        storage.list_rules()
    )
