# -*- coding: utf-8 -*-
# Copyright 2016 Objectif Libre
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#
# @author: Stéphane Albert
#
import os

from gabbi.handlers import base


class EnvironStoreHandler(base.ResponseHandler):
    """Hackish response handler used to store data in environment.

    Store data into an environment variable to implement some kind of variable
    use in gabbi.
    """
    test_key_suffix = 'store_environ'
    test_key_value = {}

    def action(self, test, key, value=None):
        data = test.content_handlers[0].extract_json_path_value(
            test.response_data, value)
        os.environ[key] = test.replace_template(data)
